package com.citygis.common.boot.util;

import java.io.File;
import java.util.List;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.citygis.common.boot.constant.constant;

/**
 * 目录工具类
 */
public final class DirectoryUtil {

    private static final Logger LOG = LoggerFactory.getLogger(DirectoryUtil.class);

    /**
     * 判断目录是否存在，如果不存在则创建新的目录
     *
     * @param dir 目录字符串
     */
    public static void existsAndNew(String dir) {
        if (!exists(dir)) {
            createDirectory(dir);
        }
    }

    /**
     * 判断是否存在目录
     *
     * @param dir 目录字符串
     * @return 目录是否存在
     */
    public static boolean exists(String dir) {
        if (StringUtil.isNullOrEmpty(dir)) {
            return false;
        }
        File file = new File(dir);
        if (file.exists()) {
            if (file.isDirectory()) {
                return true;
            } else {
                LOG.error(dir + constant.DIRECTORY.IS_NOT_VALID_DIRECTORY);
                return false;
            }
        }
        LOG.error(dir + constant.DIRECTORY.IS_NOT_EXIST);
        return false;
    }

    /**
     * 创建目录
     *
     * @param path 目录字符串
     */
    public static void createDirectory(String path) {
        File file = new File(path);
        if (file.exists()) {
            return;
        }
        file.mkdirs();
    }

    /**
     * 删除子目录和子文件，默认不删除根目录。
     *
     * @param rootDir 根目录字符串
     */
    public static void deleteSubDirAndFiles(String rootDir) {
        deleteSubDirAndFiles(rootDir, false);
    }

    /**
     * 删除子目录和子文件
     *
     * @param rootDir    根目录
     * @param deleteRoot 是否删除根目录
     */
    public static void deleteSubDirAndFiles(String rootDir, boolean deleteRoot) {
        File file = new File(rootDir);
        if (!file.exists()) {
            LOG.info(rootDir + constant.DIRECTORY.IS_NOT_EXIST);
            return;
        }
        if (!file.isDirectory()) {
            LOG.info(rootDir + constant.DIRECTORY.IS_NOT_VALID_DIRECTORY);
            return;
        }
        if (deleteRoot) {
            deleteFile(file);
        } else {
            File[] subFiles = file.listFiles();
            for (File f : Objects.requireNonNull(subFiles)) {
                deleteFile(f);
            }
        }
    }

    /**
     * 删除文件
     *
     * @param file 文件路径字符串
     */
    public static void deleteFile(File file) {
        if (!file.exists()) {
            return;
        }
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (File f : Objects.requireNonNull(files)) {
                deleteFile(f);
            }
        }
        if (!file.delete()) {
            LOG.warn(constant.DIRECTORY.DIRECTORY_CLEAN_FAIL, file.getAbsolutePath());
        }
    }

    /**
     * 获取目录下所有文件
     *
     * @param directory 目录字符串
     * @return 文件集合
     */
    public static File[] getFiles(String directory) {
        File file = new File(directory);
        if (!file.exists()) {
            LOG.error(directory + constant.DIRECTORY.IS_NOT_EXIST);
            return null;
        }
        if (!file.isDirectory()) {
            return new File[] { file };
        }
        return file.listFiles();
    }

    /**
     * 获取指定后缀的文件集合
     *
     * @param directory 目录字符串
     * @param suffix    后缀，如：.txt，.pdf等
     * @return 文件集合
     */
    public static File[] getFiles(String directory, String suffix) {
        File file = new File(directory);
        if (!file.exists()) {
            LOG.error(directory + constant.DIRECTORY.IS_NOT_EXIST);
            return null;
        }
        if (!file.isDirectory()) {
            return new File[] { file };
        }
        return file.listFiles((dir, name) -> name.toLowerCase().endsWith(suffix.toLowerCase()));
    }

    /**
     * 获取指定后缀的文件集合
     *
     * @param directory 目录字符串
     * @param suffixes  后缀数组，如：.txt、.pdf等，带有点号
     * @return 文件集合
     */
    public static File[] getFiles(String directory, List<String> suffixes) {
        File file = new File(directory);
        if (!file.exists()) {
            LOG.error(directory + constant.DIRECTORY.IS_NOT_EXIST);
            return null;
        }
        if (!file.isDirectory()) {
            return new File[] { file };
        }
        return file.listFiles((dir, name) -> suffixes.contains(PathUtil.getExtension(name, true)));
    }

    /**
     * 通过正则表达式检索匹配文件集合
     *
     * @param directory 目录
     * @param regex     正则表达式，the regular expression to which this string is to be
     *                  matched。
     *                  例如：检索后缀为.zip和.rar，则输入：.*(.zip|.rar)
     * @return 文件集合
     */
    public static File[] getFilesByRegex(String directory, String regex) {
        File file = new File(directory);
        if (!file.exists()) {
            LOG.error(directory + constant.DIRECTORY.IS_NOT_EXIST);
            return null;
        }
        if (!file.isDirectory()) {
            return new File[] { file };
        }
        return file.listFiles((dir, name) -> name.matches(regex));
    }

    /**
     * 获取文件路径列表
     *
     * @param directory 目录
     * @param regex     正则表达式，the regular expression to which this string is to be
     *                  matched。
     *                  例如：检索后缀为.zip和.rar，则输入：.*(.zip|.rar)
     * @return 文件路径列表
     */
    public static String[] getFilesPathByRegex(String directory, String regex) {
        File file = new File(directory);
        if (!file.exists()) {
            LOG.error(directory + constant.DIRECTORY.IS_NOT_EXIST);
            return null;
        }
        if (!file.isDirectory()) {
            return new String[] { file.getAbsolutePath() };
        }
        return file.list((dir, name) -> name.matches(regex));
    }

    /**
     * 获取父目录对象
     *
     * @param path 目录或文件路径
     * @return 父目录对象
     */
    public static File getParentFile(String path) {
        File file = new File(path);
        if (file.exists()) {
            return file.getParentFile();
        }
        LOG.warn(path + constant.DIRECTORY.IS_NOT_EXIST);
        return null;
    }

    /**
     * 获取父目录路径
     *
     * @param path 目录或文件路径
     * @return 父目录路径
     */
    public static String getParent(String path) {
        File file = new File(path);
        if (file.exists()) {
            return file.getParent();
        }
        LOG.warn(path + constant.DIRECTORY.IS_NOT_EXIST);
        return null;
    }
}