package com.citygis.common.boot.util;

import java.util.Arrays;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Predicate;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.citygis.common.boot.constant.constant;
import com.citygis.common.boot.enums.SystemStatusCodeEnum;
import com.citygis.common.boot.exception.BusinessException;

/**
 * 枚举工具类
 */
public final class EnumUtil {
    private final static Logger LOG = LoggerFactory.getLogger(EnumUtil.class);

    /**
     * 判断值是否是枚举有效值
     *
     * @param clz   类型
     * @param value 值
     * @return 是否是枚举有效值
     */
    public static <T> boolean isDefined(Class<T> clz, Object value) {
        if (null == value) {
            throw new BusinessException(
                    SystemStatusCodeEnum.ERROR, constant.ENUM.VALUE_NOT_BE_NULL);
        }
        if (!clz.isEnum()) {
            throw new BusinessException(
                    SystemStatusCodeEnum.ERROR, constant.ENUM.CLASS_NOT_BE_ENUM_OBJECT);
        }
        T[] objects = clz.getEnumConstants();
        if (null == objects) {
            LOG.info(constant.ENUM.NO_MATCH_ENUM_OBJECT);
            return false;
        }
        for (T sub : objects) {
            if (((Enum<?>) sub).ordinal() == Integer.parseInt(value.toString())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 根据枚举名称判断是否是一个指定类型的有效枚举对象
     *
     * @param clz  类型
     * @param name 枚举名称
     * @return 是否是一个指定类型的有效枚举对象
     */
    public static <T> boolean isDefinedByName(Class<T> clz, String name) {
        if (!clz.isEnum()) {
            throw new BusinessException(SystemStatusCodeEnum.ERROR, constant.ENUM.CLASS_NOT_BE_ENUM_OBJECT);
        }
        T[] objects = clz.getEnumConstants();
        if (null == objects) {
            LOG.info(constant.ENUM.NO_MATCH_ENUM_OBJECT);
            return false;
        }
        for (T sub : objects) {
            if (((Enum<?>) sub).name().equalsIgnoreCase(name)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 根据名字获取枚举对象
     *
     * @param type 类型
     * @param name 枚举名称
     * @return 枚举对象
     */
    public static <T extends Enum<T>> T getEnumObjByName(Class<T> type, String name) {
        if (!type.isEnum()) {
            throw new BusinessException(SystemStatusCodeEnum.ERROR, constant.ENUM.CLASS_NOT_BE_ENUM);
        }
        return Enum.valueOf(type, name);
    }

    /**
     * 根据枚举值获取枚举对象
     *
     * @param type  类型
     * @param value 枚举值
     * @return 枚举对象
     */
    public static <T extends Enum<T>> T getEnumObjByValue(Class<T> type, int value) {
        if (!type.isEnum()) {
            throw new BusinessException(SystemStatusCodeEnum.ERROR, constant.ENUM.CLASS_NOT_BE_ENUM);
        }
        return getEnumObject(type, e -> e.ordinal() == value);
    }

    /**
     * 根据条件获取枚举对象
     *
     * @param type      枚举类
     * @param predicate 筛选条件
     * @return 枚举对象
     */
    public static <T extends Enum<T>> T getEnumObject(Class<T> type, Predicate<T> predicate) {
        if (!type.isEnum()) {
            throw new BusinessException(SystemStatusCodeEnum.ERROR, constant.ENUM.CLASS_NOT_BE_ENUM);
        }
        Map<Class<T>, Object> map = new ConcurrentHashMap<>();
        Object obj = map.get(type);
        T[] ts = null;
        if (obj == null) {
            ts = type.getEnumConstants();
            map.put(type, ts);
        } else {
            ts = (T[]) obj;
        }
        Optional<T> optional = Arrays.stream(ts).filter(predicate).findAny();
        return optional.orElse(null);
    }
}
