package com.citygis.common.boot.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.nio.channels.FileChannel;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.citygis.common.boot.constant.constant;

/**
 * 文件工具类
 */
public final class FileUtil {
    private static final Logger LOG = LoggerFactory.getLogger(FileUtil.class);

    /**
     * 判断文件是否存在
     *
     * @param path 文件路径
     * @return 文件是否存在
     */
    public static boolean exists(String path) {
        File file = new File(path);
        return file.exists();
    }

    /**
     * 读取小文本内容
     *
     * @param fileName 文件路径
     * @return 文本内容
     */
    public static String readAllText(String fileName) {
        File file = new File(fileName);
        return readAllText(file);
    }

    /**
     * 读取小文本内容
     *
     * @param file 文件对象
     * @return 文本内容
     */
    public static String readAllText(File file) {
        cn.hutool.core.io.file.FileReader fileReader = new cn.hutool.core.io.file.FileReader(file);
        return fileReader.readString();
    }

    /**
     * 写文本到指定文件
     *
     * @param path   文件路径
     * @param append 是否追加到文件后
     */
    public static void writeText(Object content, String path, boolean append) {
        File file = new File(path);
        try {
            FileWriter fw = new FileWriter(file, append);
            PrintWriter pw = new PrintWriter(fw);
            pw.print(content);
            pw.flush();
            fw.flush();
            pw.close();
            fw.close();
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
        }
    }

    /**
     * 删除文件
     *
     * @param fullPath 文件全路径
     */
    public static void delete(String fullPath) {
        File file = new File(fullPath);
        if (file.exists()) {
            file.delete();
        }
    }

    /**
     * 拷贝源文件到目的文件
     *
     * @param sourceFileName 源文件
     * @param destFileName   产生新文件
     */
    public static void copy(String sourceFileName, String destFileName) throws IOException {
        FileChannel inputChannel = null;
        FileChannel outputChannel = null;
        try {
            inputChannel = new FileInputStream(sourceFileName).getChannel();
            outputChannel = new FileOutputStream(destFileName).getChannel();
            outputChannel.transferFrom(inputChannel, 0, inputChannel.size());
        } finally {
            Objects.requireNonNull(inputChannel).close();
            Objects.requireNonNull(outputChannel).close();
        }
    }

    /**
     * 复制文件流到目标文件
     *
     * @param stream       文件流
     * @param destFilePath 目标文件
     */
    public static void copy(InputStream stream, String destFilePath) throws IOException {
        Files.copy(stream, Paths.get(destFilePath));
    }

    /**
     * 获取文件长度
     *
     * @param filePath 文件路径
     * @return 单位：bytes
     */
    public static long getLength(String filePath) {
        File file = new File(filePath);
        if (file.exists() && file.isFile()) {
            return file.length();
        }
        LOG.info(constant.FILE.PATH_NOT_EXIST_OR_INVALID, filePath);
        return 0L;
    }

    /**
     * 获取文件字节数组
     *
     * @param path 文件
     * @return 文件字节数组
     */
    public static byte[] readAllBytes(String path) throws IOException {
        return Files.readAllBytes(Paths.get(path));
    }

    /**
     * 移动文件
     *
     * @param sourceFileName 源文件全路径
     * @param destFileName   目标文件全路径
     * @param option         移动策略
     */
    public static void move(String sourceFileName, String destFileName, StandardCopyOption option) throws IOException {
        Files.move(Paths.get(sourceFileName), Paths.get(destFileName), option);
    }

    /**
     * 移动文件，如果存在则覆盖。
     *
     * @param sourceFileName 源文件全路径
     * @param destFileName   目标文件全路径
     */
    public static void move(String sourceFileName, String destFileName) throws IOException {
        Files.move(Paths.get(sourceFileName), Paths.get(destFileName), StandardCopyOption.REPLACE_EXISTING);
    }

    /**
     * 判断目录下是否存在指定后缀的文件
     *
     * @param directory    目标目录
     * @param suffixRegexp 正则表达式
     * @return 是否存在指定后缀的文件
     */
    public static boolean exist(String directory, String suffixRegexp) {
        return cn.hutool.core.io.FileUtil.exist(directory, suffixRegexp);
    }
}