package com.citygis.common.boot.util;

import com.alibaba.fastjson2.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * JSON转换工具类
 */
public final class JSONUtil {

    private final static Logger LOG = LoggerFactory.getLogger(JSONUtil.class);

    /**
     * 将java对象转换成JSON字符串，若对象为null，那么返回null。默认也序列化空属性。
     *
     * @param o 需要被转换的java对象
     * @return json格式的字符串
     */
    public static String serializeObject(Object o) {
        return JSONObject.toJSONString(o, JSONWriter.Feature.WriteMapNullValue);
    }

    /**
     * 序列化对象（使用toString方法）
     *
     * @param o 对象
     * @return JSON字符串
     */
    public static String serializeObjectUsingToString(Object o) {
        return JSONObject.toJSONString(o, JSONWriter.Feature.WriteEnumUsingToString);
    }

    /**
     * 转换为JSON对象
     *
     * @param content 转换内容
     * @return JSON对象
     */
    public static JSONObject parseObject(String content) {
        if (StringUtil.isNullOrEmpty(content)) {
            return null;
        }
        return JSONObject.parseObject(content);
    }

    /**
     * 转换为JSON数组
     *
     * @param content 转换内容
     * @return JSON数组
     */
    public static JSONArray parseArray(String content) {
        if (StringUtil.isNullOrEmpty(content)) {
            return null;
        }
        return JSONArray.parseArray(content);
    }

    /**
     * 校验是否JSON对象或JSON数组文本内容
     *
     * @param content 字符串
     * @return 是否JSON对象或JSON数组文本内容
     */
    public static boolean isJson(String content) {
        try {
            if (StringUtil.isNullOrEmpty(content)) {
                return false;
            }
            // 特殊处理
            if ("null".equalsIgnoreCase(content)) {
                return false;
            }
            return JSONValidator.from(content).validate();
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
            return false;
        }
    }

    /**
     * 校验是否JSON数组
     *
     * @param content 字符串
     * @return 是否JSON数组
     */
    public static boolean isJsonArray(String content) {
        try {
            return JSONValidator.from(content).validate();
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
            return false;
        }
    }

    /**
     * 校验是否JSON对象
     *
     * @param content 字符串
     * @return 是否JSON对象
     */
    public static boolean isJsonObject(String content) {
        return JSONValidator.from(content).validate();
    }

    /**
     * 反序列化JSON对象
     *
     * @param content 字符串
     * @param type    类型
     * @return 指定类型对象实例
     */
    public static <T> T deserializeObject(String content, Class<T> type) {
        return JSONObject.parseObject(content, type);
    }

    /**
     * 用于反序列化复杂嵌套对象，例如Map<String, Property>
     *
     * @param content 字符串
     * @param type    类型
     * @return 指定类型对象实例
     */
    public static <T> T deserializeObject(String content, TypeReference<T> type) {
        return JSONObject.parseObject(content, type);
    }

    /**
     * 反序列化JSON数组对象
     *
     * @param content 字符串
     * @param type    类型
     * @return 指定类型数组实例
     */
    public static <T> List<T> deserializeArray(String content, Class<T> type) {
        return JSON.parseArray(content, type);
    }

    /**
     * 反序列化对象
     *
     * @param content 字符串
     * @return 对象实例
     */
    public static Object deserializeObject(String content) {
        return JSON.parse(content);
    }
}
